'Build Version: 3.0.118.0
'**********************************************************************
' THIS CODE AND INFORMATION IS PROVIDED TO YOU FOR YOUR REFERENTIAL 
' PURPOSES ONLY, AND IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
' WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE, 
' AND MAY NOT BE REDISTRIBUTED IN ANY MANNER.
' 
' Copyright (c) 2004  Microsoft Corporation.  All rights reserved.
'**********************************************************************

Sub ADCreateCustomer()
' **********************************************************
' Function ADCreateCustomer <none>
' Description: Creates the customer in Active Directory
' Creates the OU
' Creates the User Accounts
' Sets the required Properties
' 
' In: none
' Out: none
' **********************************************************
' Bind to the root of the Domain
Set oRoot = GetObject("LDAP://RootDSE")
' Connect to the defaultNamingContext
Set oDefaultNC = GetObject("LDAP://" & oRoot.Get("defaultNamingContext"))
' Bind to the customer Root Container
Set oCustomerContainer = GetObject("LDAP://" & constCustomerContainer & "," & oRoot.Get("defaultNamingContext"))

write "Bound to " & oCustomerContainer.AdsPath, Err_INFO	
' Create the OU for the new customer
Set oNewContainer = oCustomerContainer.Create("organizationalUnit","ou=" & strCustomer)
' Commit the changes to AD
oNewContainer.Setinfo

Write "Creating Group Accounts.",Err_INFO
' Create the admins Group for the customer
Set oAdminGroup = oNewContainer.Create("group","cn=Admins@" & strCustomer)
' set the SAM AccountName for the group
' make sure it is not longer then 20 Characters and does Not
' start or end with a space (chr(32))
oAdminGroup.SamAccountName = Left("adm_" & Trim(strCustomer),20)
' Commit the Changes to AD
oAdminGroup.SetInfo
Write "Created Admins@" & strCustomer & " Group.",Err_INFO

' Create the users Group for the customer
Set oUserGroup = oNewContainer.Create("group","cn=Users@" & strCustomer)
' set the SAM AccountName for the group
oUserGroup.SamAccountName = Left("usr_" & Trim(strCustomer),20)
' Commit the Changes to AD
oUserGroup.SetInfo
Write "Created Users@" & strCustomer & " Group.",Err_INFO

Write "Creating User Accounts.",Err_INFO
' Create the IUSR_ Account for the new customer
Set oNewUser = oNewContainer.Create("user","cn=IUSR_" & strCustomer)
' Set the SAM Account Name (Alias)
oNewUser.SamAccountName = Left("I_" & Trim(strCustomer),20)
' Commit the Changes to AD
oNewUser.SetInfo
' Enable the user
oNewUser.AccountDisabled = False
' Set the Password
oNewUser.Setpassword strPassword
' Commit the Changes to AD
oNewUser.SetInfo
Write "Created User " & oNewUser.Adspath,Err_INFO

' Create the UNC Connection Account
' we always create the Account so we do not have to touch AD
' in case the customer needs to be migrated later
Set oNewUser = oNewContainer.Create("user","cn=UNC_" & strCustomer)
' Set the SAM Account Name (Alias)
oNewUser.SamAccountName = Left("U_" & Trim(strCustomer),20)
' Commit the Changes to AD
oNewUser.SetInfo
' Enable the user
oNewUser.AccountDisabled = False
' Set the Password
oNewUser.Setpassword strPassword
' Commit the Changes to AD
oNewUser.SetInfo

Write "Created User " & oNewUser.Adspath,Err_INFO
' Create the FTP User for the new customer
Set oNewUser = oNewContainer.Create("user","cn=FTP_" & strCustomer)
' Set the SAM Account Name (Alias)
oNewUser.SamAccountName = Left("F_" & Trim(strCustomer),20)
' Commit the Changes to AD
oNewUser.SetInfo
' Enable the user
oNewUser.AccountDisabled = False
' Set the Password
oNewUser.Setpassword strPassword
' Commit the Changes to AD
oNewUser.SetInfo
' Now add the User to the Users Group
' the AddToGroupAD() Function is located in this script
varResult = AddToGroupAD(oUserGroup, oNewUser)
Write "Created User " & oNewUser.Adspath,Err_INFO

' Create the Admin User Account for the Customer
Set oNewUser = oNewContainer.Create("user","cn=Admin@" & strCustomer)
' Set the SAM Account Name (Alias)
oNewUser.SamAccountName = strAdminSAMName
' Commit the Changes to AD
oNewUser.SetInfo
' Set the User Principal Name for the Logon
oNewUser.UserPrincipalName = "Admin@" & strCustomer
' enable the user
oNewUser.AccountDisabled = False
' Set the Password
oNewUser.Setpassword strPassword
' Set the FTP Properties
onewUSer.Put("msIIS-FTPDir"), strCustomer
onewUSer.Put("msIIS-FTPRoot"), strpath & "\"
' Commit the Changes to AD
oNewUser.SetInfo
' Now add the user to the Admins Group
varResult = AddToGroupAD(oAdminGroup,oNewUser)
Write "Created User " & oNewUser.Adspath,Err_INFO

' Now that all the users and groups are set up we need to reset the security 
' on the OU Object
SetADSecurity oNewContainer

Write "Done Creating AD Objects for Customer " & strCustomer,Err_NONE

End Sub

Function AddToGroupAD (oGroup,oUser)
' **********************************************************
' Function AddToGroupAD (oGroup,oUser) 
' Description: Adds a user to a group in Active Driectory
' Details about the Functions used can be found at
' http://msdn.microsoft.com/library/default.asp?url=/library/en-us/adsi/adsi/iadsaccesscontrolentry_property_methods.asp
' 
' In:	oGroup	....	the Group Object the user will be added to.
'		oUser	....	the UserObject to be added to the group.
'		
' Out: 	AddToGroupAD	the result code for the operation
'						usually 0 if successfull or an
'						ADSI, LDAP or WIN32 Error.
' **********************************************************

' Do a quick Sanity check if the user and the Group are in the same
' OU otherwise do not add the user and return an error.
If oGroup.Parent = oUser.Parent Then
	Write " Adding user " & oUser.Name & " to group " & oGroup.Name, Err_INFO
	oGroup.Add oUser.ADSPath
	' Commit Changes to AD
	oGroup.SetInfo
	varResult = Err.Number
Else
	varResult = 99
End If
AddToGroupAD = varResult
End Function

Function SetADSecurity(oOU)
' **********************************************************
' Function SetADSecurity (oOU)
' Description: 	Setup the ACLs in Active Driectory for the new 
'				customer
' 
' In:	oOU	....		the OU Object the ACL will be modified on
'		
' Out: 	none
' **********************************************************
' Set Public Constants
Const SE_DACL_PROTECTED				= &H1000
Const SE_SACL_PROTECTED				= &H2000
	' Get the SamAccountNames for the Users and Admin Groups
	strUserGroupName = oUserGroup.SamAccountName
	strAdminGroupName = oAdminGroup.SamAccountName
	Set securityDescriptor = oOU.Get("ntSecurityDescriptor")
	Set oDACL = securityDescriptor.DiscretionaryAcl
	
	Write "DACL before Editing", Err_Info
	For Each a In oDACL
		Write a.Trustee & " " & a.Flags, Err_INfo
	next
	
	Set oACE = CreateObject("AccessControlEntry")
	DeleteAce oDACL, "NT AUTHORITY\Authenticated Users"
	DeleteAce oDACL, strAdminGroupName
	'	Reset AllUsers
	DeleteAce oDACL, strUserGroupName
	
	' Allow Users from within this OU to view other members in the same Org
	' CHEAT Sheet: AddAce(TrusteeName, gAccessMask, gAceType, gAceFlags, gFlags, gObjectType, gInheritedObjectType)
	AddAce oDACL, strUserGroupName, 131092, 0, 2, 0, 0, 0
	
	'** Full Control of Organizational Units
	' Organizational Unit, (Full Control)
	AddAce oDACL, strAdminGroupName, 983551, 5, 10, 2, 0,"{BF967AA5-0DE6-11D0-A285-00AA003049E2}"
		

	' This object and all child objects, (Create User Objects, Delete User Objects)
	AddAce oDACL, strAdminGroupName, 3, 5, 2, 1, "{BF967ABA-0DE6-11D0-A285-00AA003049E2}", 0
	
	'** Full Control of User Objects
	' User Object, (Full Control)
	AddAce oDACL, strAdminGroupName, 983551, 5, 10, 2, 0, "{BF967ABA-0DE6-11D0-A285-00AA003049E2}"
	
	'** Reset User Passwords
	' User Objects, (Reset Password)
	AddAce oDACL, strAdminGroupName, 256, 5, 10, 3, "{00299570-246D-11D0-A768-00AA006E0529}", "{BF967ABA-0DE6-11D0-A285-00AA003049E2}"
	
	'** Create & Delete Groups
	' This object and all child objects, (Create Group Objects, Delete Group Objects)
	AddAce oDACL, strAdminGroupName, 3, 5, 2, 1, "{BF967A9C-0DE6-11D0-A285-00AA003049E2}", 0
	
	'** Full Control of Groups
	' Group Objects, (Full Control)
	AddAce oDACL, strAdminGroupName, 983551, 5, 10, 2, 0, "{BF967A9C-0DE6-11D0-A285-00AA003049E2}"
	
	'** Modify the Membership of Groups
	AddAce oDACL, strAdminGroupName, 48, 5, 10, 3, "{BF9679C0-0DE6-11D0-A285-00AA003049E2}", "{BF967A9C-0DE6-11D0-A285-00AA003049E2}"
	
	'** Create & Delete Organizational Units
	' This object and all child objects, (Create Organizational Unit, Delete Organizational Unit)
	AddAce oDACL, strAdminGroupName, 3, 5, 2, 1, "{BF967AA5-0DE6-11D0-A285-00AA003049E2}", 0
	
	
	'** Write Properties to the Root OU Object Only
	' This Object, (Write Properties)
	AddAce oDACL, strAdminGroupName, 32, 0, 0, 0, 0, 0
	'Set oDACL = ReorderACL(oDACL,FALSE)
	Write "DACL after Editing", Err_Info
	For Each a In oDACL
		Write a.Trustee & " " & a.Flags, Err_INFO
	next
	'** Add New Dacl to the Security Descriptor
	SecurityDescriptor.discretionaryAcl = oDACL
	oOU.put "ntSecurityDescriptor", SecurityDescriptor
	' Commit the changes to AD
	oOU.SetInfo
End Function

Function DeleteAce(oDACL, TrusteeName)
'*******************************************************************
' Function DeleteAce(oDACL, TrusteeName)
'
' Purpose: Deletes an ACE from an existing DACL
' Details about the Functions used can be found at
' http://msdn.microsoft.com/library/default.asp?url=/library/en-us/adsi/adsi/iadsaccesscontrolentry_property_methods.asp
' Input:   	oDACL						Access Control List (Object)
'				TrusteeName					Name of Trustee to add (String)
'				
' Output:  None
'*******************************************************************
		' iterate throught the ACE Collection 
		' to find the Ace we are looking for
		For Each oAce in odacl
			if (oAce.Trustee = TrusteeName) then
				Write "Found and Deleted: " & TrusteeName, Err_INFO
				' Remove the ACE from the DACL
				oDACL.RemoveAce(oAce)
			Else
				'Write TrusteeName & " not Found", Err_INFO
			End if
		Next		
End Function


Function AddAce(oDACL, sTrusteeName, vAccessMask, vAceType, vAceFlags, vFlags, vObjectType, vInheritedObjectType)
'**************************************************************
' Function AddAce(oDACL, TrusteeName, vAccessMask, vAceType,_
'				 vAceFlags, vFlags, vObjectType, vInheritedObjectType)
'
' Purpose: Adds and ACE to an existing acl
' Details about the Functions used can be found at
' http://msdn.microsoft.com/library/default.asp?url=/library/en-us/adsi/adsi/iadsaccesscontrolentry_property_methods.asp
' Input:   	dacl						Access Control List (Object)
'				TrusteeName					Name of Trustee to add (String)
'				vAccessMask					AccessMask (String)
'				vAceType					AceType ()
'				vAceFlags					AceFlags ()
'				vFlags						Flags ()
'				vObjectType					ObjectType ()
'				vInheritedObjectType		InheritedObjectType ()
'				
' Output:  None
'*******************************************************************
	Dim oNewAce
	Write "Adding ACE for " & sTrusteeName, Err_Info
	' Create the new ACE Object
	Set oNewAce = CreateObject("AccessControlEntry")
	' Put the Access Mask in
	oNewAce.AccessMask = vAccessMask
	' Set the ACE Type
	oNewAce.AceType = vAceType
	' Set the inheritance Flags
	oNewAce.AceFlags = vAceFlags
	' Set the Flags
	oNewAce.Flags = vFlags
	' Set the Trustee
	oNewAce.Trustee = sTrusteeName

		'Check to See if ObjectType needs to be set
		If cStr(vObjectType) <> "0" then
			oNewAce.ObjectType = vObjectType
		End If

		'Check to See if InheritedObjectType needs to be set
		If cStr(vInheritedObjectType) <> "0" then
			oNewAce.InheritedObjectType = vInheritedObjectType
		End If
	oDACL.AddAce oNewAce
	
	' Kill Objects
	Set oNewAce = Nothing
End Function

'********************************************************************
'*
'* Function ReorderACL(objDACL)
'*
'* Purpose: Reorders a DACL properly
'* Input:   	objDACL						Access Control List (Object)
'*				bMakeExplicit				Determinse if whether or not inherited
'*											ACE Settings should be stripped (boolean)
'*				
'* Output:  Object - Reordered DACL
'*
'********************************************************************
Function ReorderACL(objDACL, bMakeExplicit)
	' Dim Objects
	Dim ImpDenyDacl, ImpDenyObjectDacl, InheritedDacl, ImpAllowDacl, ImpAllowObjectDacl
	Dim objSD, newDACL
	
	' Dim Other Variables
	Dim ace
	
	' Set Constants
	Const ADS_ACEFLAG_INHERITED_ACE				= &H10
	Const ADS_ACETYPE_ACCESS_ALLOWED			= &H0
	Const ADS_ACETYPE_ACCESS_DENIED				= &H1
	Const ADS_ACETYPE_ACCESS_ALLOWED_OBJECT		= &H5
	Const ADS_ACETYPE_ACCESS_DENIED_OBJECT		= &H6
	
	' Create the New DACL
	Set objSD = CreateObject("SecurityDescriptor")
	
	' Create the ACL Objects
	Set newDACL					= CreateObject("AccessControlList")
	Set ImpDenyDacl				= CreateObject("AccessControlList")
	Set ImpDenyObjectDacl		= CreateObject("AccessControlList")
	Set InheritedDacl			= CreateObject("AccessControlList")
	Set ImpAllowDacl			= CreateObject("AccessControlList")
	Set ImpAllowObjectDacl		= CreateObject("AccessControlList")
	
	    ' Loop through the Original Dacl.
    For Each ace In objDACL
    
   
            Select Case ace.AceType
                Case ADS_ACETYPE_ACCESS_ALLOWED
                    ImpAllowDacl.AddAce ace
                Case ADS_ACETYPE_ACCESS_DENIED
                    ImpDenyDacl.AddAce ace
                Case ADS_ACETYPE_ACCESS_ALLOWED_OBJECT
                    ImpAllowObjectDacl.AddAce ace
                Case ADS_ACETYPE_ACCESS_DENIED_OBJECT
                    ImpDenyObjectDacl.AddAce ace
                Case Else
                    'Bad Ace, but let's just leave it out for now.
            End Select
    
    Next
	'
	' Combine the ACEs in the Proper Order
	'	Implicit Deny
	'	Implicit Deny Object
	'	Implicit Allow
	'	Implicit Allow Object
	'	Inherited ACEs
	'
	
	' Implicit Deny
		For Each ace in ImpDenyDacl
			newDACL.AddAce ace
		Next
		
	' Implicit Deny Object
		For Each ace in ImpDenyObjectDacl
			newDACL.AddAce ace
		Next
		
	' Implicit Allow
		For Each ace in ImpAllowDacl
			newDACL.AddAce ace
		Next
		
	' Implicit Allow Object
		For Each ace in ImpAllowObjectDacl
			newDACL.AddAce ace
		Next
		
	' Inherited ACEs
		For Each ace in InheritedDacl
			newDACL.AddAce ace
		Next
	
	'Set the Appropriate revision level for the DACL
	newDACL.AclRevision = objDACL.AclRevision
	
	' Return Properly Ordered DACL
	Set ReorderACL = newDACL
	
	' Kill Objects
	Set newDACL					= Nothing
	Set InheritedDacl			= Nothing
	Set ImpAllowObjectDacl		= Nothing
	Set ImpAllowDacl			= Nothing
	Set ImpDenyObjectDacl		= Nothing
	Set ImpDenyDacl				= Nothing
	Set objSD					= Nothing
	
End Function
